import { SSTMenuItem } from './sst-menu.interface';

import { Component, Input, ViewEncapsulation, OnInit, ChangeDetectionStrategy } from '@angular/core';
import { faDotCircle, faAngleDown, faAngleUp } from '@fortawesome/free-solid-svg-icons';
import { Observable } from 'rxjs';
import { SSTMenuService } from './sst-menu.service';
import { ActivatedRoute, RouterState, Router, NavigationEnd } from '@angular/router';

@Component({
  selector: 'sst-menu',
  template: `
    <sst-menu-list class="sst-menu" [items]="sstMenuService.menuItems$ | async" [currentPath]="currentPath"></sst-menu-list>
  `,
  styleUrls: ['./sst-menu.component.scss'],
  encapsulation: ViewEncapsulation.None
})
export class SSTMenuComponent {

  items$: Observable<SSTMenuItem[]>;
  currentPath: string;

  constructor(
    public sstMenuService: SSTMenuService,
    public route: Router
  ) {
    this.route.events.subscribe( event => {
      if (event instanceof NavigationEnd) {
          this.currentPath  = event.url;
      }
    });
  }

}

@Component({
  selector: 'sst-menu-list',
  template: `
    <ul class="sst-menu-list">
      <ng-template ngFor let-item [ngForOf]="items">
        <li>
          <sst-menu-item *ngIf="!isDropdown(item)" [parent]="parent" [item]="item"
            [currentPath]="currentPath">{{item.label}}</sst-menu-item>
          <sst-menu-dropdown *ngIf="isDropdown(item)" [parent]="parent" [item]="item"
            [currentPath]="currentPath">{{item.label}}</sst-menu-dropdown>
        </li>
      </ng-template>
    </ul>
  `,
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class SSTMenuListComponent {
  @Input()
  items: SSTMenuItem[];

  @Input() currentPath: string;

  @Input()
  parent = ['/'];

  public isDropdown(item) {
    return item.children ? true : false;
  }
}

@Component({
  selector: 'sst-menu-item',
  template: `
    <a [routerLink]="mountLink(item)" sstHideNav [class.active]="isActive(mountLink(item).join('/'))">
      <fa-icon [icon]="item.icon"></fa-icon>
      <span><ng-content></ng-content></span>
    </a>
  `,
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class SSTMenuItemComponent implements OnInit {

  @Input() currentPath: string;

  faDotCircle = faDotCircle;

  @Input()
  item: SSTMenuItem;

  @Input()
  parent: string[];

  link: string[] = [];

  ngOnInit() {
    // console.log(this.isActive(this.mountLink(this.item).join('/')));
  }

  mountLink(item) {
    return item.path[0] === '' ? this.parent : [...this.parent, ...item.path];
  }

  isActive(path: string): boolean {
    const exp = path.replace('//', '/').replace('/', '\/');
    const regx = new RegExp(`^${exp}$`, 'i');
    // console.log(regx, this.currentPath);
    return regx.test(this.currentPath);
  }
}

@Component({
  selector: 'sst-menu-dropdown',
  template: `
    <a (click)="toggleExpand($event)" [class.expanded]="expanded" href="#">
      <fa-icon [icon]="item.icon"></fa-icon>
      <span><ng-content></ng-content></span>
      <fa-icon class="expand-icon" [icon]="expandedIcon()"></fa-icon>
    </a>
    <sst-menu-list class="list" [items]="item.children" [parent]="mountParent(item)"
      [currentPath]="currentPath" [class.expanded]="expanded"></sst-menu-list>
  `,
  styles: [
    `.list { display: none; }`,
    `.list.expanded { display: block; }`
  ],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class SSTMenuDropdownComponent implements OnInit {

  faDotCircle = faDotCircle;

  @Input() currentPath: string;

  @Input()
  item: SSTMenuItem;

  @Input()
  expanded = false;

  @Input()
  parent = ['/'];

  ngOnInit() {
    this.expanded = this.isExpanded( this.mountParent(this.item).join('/') );
  }

  expandedIcon() {
    return this.expanded ? faAngleUp : faAngleDown;
  }

  isExpanded(path: string): boolean {
    const exp = path.replace('//', '/').replace('/', '\/');
    const regx = new RegExp(`^${exp}`, 'i');
    return regx.test(this.currentPath);
  }

  mountParent(item) {
    return item.path[0] === '' ? this.parent : [...this.parent, ...item.path];
  }

  toggleExpand(event) {
    event.preventDefault();
    this.expanded = !this.expanded;
  }
}
